#import <UIKit/UIKit.h>

/**
 * Contains filter parameter information.
 * Filter parameter is specified by the following:
 * - name: name of the parameter to set
 * - type: type of the parameter. The system currently supports floating point and texture types. Type defines how the given value will be parsed and represented.
 * - value: value of the parameter. Value is provided as a string, regardless of type. Texture inputs are given by name of the texture. Currently the only supported texture inputs are a custom mask names defined in PipeTemplate object.
 * Provides methods for serialization and deserialization of FilterParameterTemplate instances from JSON format.
 *
 */
@interface FilterParameterTemplate:NSObject
{
    /**
     * Name of the input parameter.
     */
    @public NSString* name;

    /**
     * Value of the input parameter.
     */
    @public NSString* value;

    /**
     * Type of the input parameter.
     */
    @public NSString* type;
}
@end

/**
 * Defines a name and a paramaters list specifiying a filter.
 * Each filter must take an image as input and returns and image processed by an filtering algorithm. Optionally, a filter can take additional parameters which are stored in parameters member.
 * The engine supports the following filters:
 * - Smoothing: performs face smoothing filter on the input image
 * - Greyscale: performs greyscale conversion on the input image
 * - Threshold: performs binary thresholding operation on the input image
 * - Multply: performs multiplication of the given image with a custom mask texture specified as input parameter
 * - HighPass: performs gaussian high pass filter on the input image
 * - Blur - performs gaussian blur on the input image
 *
 * The filter applied is defined by name member.
 * Input parameters of each filter can be set using FilterParameterTemplate class
 * Provides methods for serialization and deserialization of FilterTemplate instances from JSON format.
 */
@interface FilterTemplate:NSObject
{
    /**
     * Name of the filter to apply.
     */
    @public NSString* name;

    /**
     * List of optional filter parameters.
     */
    @public NSMutableArray<FilterParameterTemplate*>* parameters;
}

/**
 * Initializes new FilterTemplate object.
 */
- (id)init;

/**
 * Returns FilterTemplate object out of JSON dictionary.
 * @param jsonDictionary JSON dictionary
 * @return deserialized FeatureTemplate object
 */
+ (FilterTemplate*) FromJSON: (NSDictionary*) jsonDictionary;


/**
 * Returns JSON dictionary out of this FilterTemplate object.
 * @return JSON representation of this FilterTemplate returned as NSMutableDictionary
 */
- (NSMutableDictionary*) ToJSON;
@end

/**
 * Contains information defining a single step in the rendering pipeline of each makeup feature element.
 * Single step of the rendering pipeline consists of a blend mode, an optional lists of preprocessing and an optional list of postprocessing filters.
 * The engine supports the following blend modes:
 * - Overlay: performs overlay blend mode
 * - AlphaBlendUseMaskAlpha: performs traditional transparency blending using binary mask as alpha channel
 * - Screen: performs screen blend mode
 *
 * Each blend is performed between the output of the preprocessing stack and the output of the previous blend or previous postprocessing stack. Used blend mode is specified via name member.
 * Specifically, the initial blend is performed between colored a mesh representing queried makeup feature and the first preprocessing stack.
 * Preprocessing filters are applied in sequence starting from raw camera feed. Postprocessing filters are applied in sequence starting from the output of blending.
 * Provides methods for serialization and deserialization of FilterQueueTemplate instances from JSON format.
 */
@interface FilterQueueTemplate:NSObject
{
    /**
     * Name of the used blend mode.
     */
    @public NSString* blend;

    /**
     * The input image for this filter queue.
     * The following values are supported:
     * <br/>
     * - video: the input image into this filter queue is the input image provided to the engine by the API consumer.
     * <br/>
     * - previous: the input image into this filter queue is the output image of the previous element in the pipeline
     * <br/>
     * <br/>
     *
     * The default value for origin is video.
     */
    @public NSString* origin;

    /**
     * List of preprocessing filters to apply.
     */
    @public NSMutableArray<FilterTemplate*> *preprocess;

    /**
     * List of postprocessing filters to apply.
     */
    @public NSMutableArray<FilterTemplate*> *postprocess;

    /**
     * If set to true the output of preprocess is used as a top layer, otherwise output from previous blend is used as a top layer for blending.
     */
    @public Boolean switchLayers;
}

/**
 * Initializes new FilterQueueTemplate object.
 */
- (id)init;

/**
 * Returns FilterQueueTemplate object out of JSON dictionary
 * @param jsonDictionary JSON dictionary
 * @return deserialized FilterQueueTemplate object
 */
+ (FilterQueueTemplate*) FromJSON: (NSDictionary*) jsonDictionary;

/**
 * Returns JSON dictionary out of this FilterQueueTemplate object
 * @return JSON representation of this FilterQueueTemplate returned NSMutableDictionary
 */
-(NSMutableDictionary*)ToJSON;
@end

/**
 * Contains a definition of a custom mask
 * Custom mask is defined by name and a reference which is used to determine the location of the image file containg the mask.
 *
 * The location for storing of custom masks is <path to platform dependent assets folder> + "/Textures/" + MaskTemnplate.texture  + "/" + FeatureTemplate.type + "/" + FeatureTemplate.area.
 *
 * The mesh custom mask should use is derived from the type of FeatureTemplate instance this MaskTemplate is contained in.
 */
@interface MaskTemplate:NSObject
{
    /**
     * Name of the custom texture used for identification purposes. Should be unique within a single FeatureTemplate instance.
     */
    @public NSString* name;

    /**
     * Location reference used to determine the location of the image file containg the mask.
     */
    @public NSString* texture;
}

/**
  * Returns MaskTemplate object out of JSON dictionary
  * @param jsonDictionary JSON dictionary
  * @return deserialized MaskTemplate object
  */
+ (MaskTemplate*) FromJSON: (NSDictionary*) jsonDictionary;


/**
 * Returns JSON dictionary out of this MaskTemplate object
 * @return JSON representation of this FeatureTemplate returned as NSMutableDictionary
 */
- (NSMutableDictionary*)ToJSON;
@end

/**
 * Stores a list of blending and filtering steps required to render a makeup feature.
 * Contains of a list of MaskTemplate objects defining custom masks which can be used as inputs in filters.
 * Contains a list of FilterQueueTemplate objects each of which define a single step in the rendering procedure.
 */
@interface PipeTemplate:NSObject
{
    /**
     * List storing custom mask definitions.
     */
    @public NSMutableArray<FilterQueueTemplate *> *filters;

    /**
     * List storing blending and filtering steps of rendering procedure of a makeup feature.
     */
    @public NSMutableArray<MaskTemplate *> *masks;
}

/**
 * Initializes new PipeTemplate.
 */
- (id)init;

/**
 * Returns PipeTemplate object out of JSON dictionary.
 * @param jsonDictionary JSON dictionary
 * @return deserialized PipeTemplate object
 */
+(PipeTemplate*)FromJSON:(NSDictionary*)jsonDictionary;

/**
 * Returns JSON dictionary out of this PipeTemplate object.
 * @return JSON representation of this PipeTemplate returned as NSMutableDictionary
 */
-(NSMutableDictionary*)ToJSON;
@end

/**
 * Stores a definition of makeup finish. Each finish is a named PipeTemplate object which can be reused with any of the features.
 * Contains code for serialization and deserialization of FinishTemplate instances from JSON format.
 * Provides methods for setup of blending and filtering procedures and parameters for finishes.
 */
@interface FinishTemplate:NSObject
{
    /**
     * Name of the finish.
     */
    @public NSString* name;

    /**
     * Transparency of the finish.
     */
    @public double transparency;

    /**
     * Rendering steps to use when rendering the finish.
     */
    @public PipeTemplate *pipeline;
}

/**
 * Returns FinishTemplate object out of JSON dictionary.
 * @param jsonDictionary JSON dictionary
 * @return deserialized FinishTemplate object
 */
+ (FinishTemplate*) FromJSON: (NSDictionary*) jsonDictionary;

/**
 * Returns FinishTemplate object out of JSON string.
 * @param JSON_data JSON string
 * @return deserialized FinishTemplate object
 */
+ (FinishTemplate*) FromJSONString: (NSString*) JSON_data;

/**
 * Creates JSON representation out of this FinishTemplate object.
 * @return JSON representation of this FinishTemplate returned as NSMutableDictionary
 */
- (NSMutableDictionary*)ToJSON;

/**
 * Creates JSON string out of FinishTemplate object.
 * @param finishTemplate FinishTemplate object
 * @return JSON representation of this FinishTemplate returned as string
 */
- (NSString*)ToJSONString;

@end

/**
 * Contains a list of finish templates.
 */
@interface FinishTemplateRepository:NSObject
{
    NSMutableArray<FinishTemplate*> *finishTemplates;
}

/**
 * Initializes new FinishTemplateRepository.
 */
- (id)init;

/**
  * Returns FinishTemplateRepository object out of JSON dictionary.
  * @param JSON_data JSON string
  * @return deserialized FinishTemplateRepository object
  */
+ (FinishTemplateRepository*)FromJSON:(NSDictionary*)JSON_data;

/**
  * Returns FinishTemplateRepository object out of JSON string.
  * @param JSON_data JSON string
  * @return deserialized FinishTemplateRepository object
  */
+ (FinishTemplateRepository*)FromJSONString:(NSString*)JSON_data;

/**
  * Returns FinishTemplate object at index i.
  * @param i index
  * @return FinishTemplate object at given index
  */
- (FinishTemplate*)Get:(int)i;

/**
 * Returns total number of stored finish templates.
 * @return the number of FinishTemplate objects in the repository
 */
- (int)GetCount;

/**
 * Adds new FinishTemplate object to the repository.
 * @param finishTemplate FinishTemplate object
 */
- (void)Add:(FinishTemplate*)finishTemplate;

/**
 * Returns JSON representation out of this FinishTemplateRepository object.
 * @return JSON representation of this FinishTemplateRepository returned as NSMutableDictionary
 */
- (NSMutableDictionary*)ToJSON;

/**
 * Returns JSON string out of this FinishTemplateRepository object.
 * @return JSON representation of this FinishTemplateRepository returned as string
 */
- (NSString*)ToJSONString;

@end


/**
 * Class used as a container storing RGBA for color values.
 * Color is stored as floating point value in 0.0 to 1.0 range.
 */
@interface Color:NSObject
{
    @public float r;
    @public float g;
    @public float b;
    @public float a;
}

/**
 * Initializes new Color object.
 */
- (id)init;

/**
 * Initializes new Color object with passed color values in float.
 * @param red float value for red channel
 * @param green float value for green channel
 * @param blue float value for blue channel
 * @param alpha float value for alpha channels
 */
- (id)initWithRed:(float)red andGreen:(float)green andBlue:(float)blue andAlpha:(float)alpha;

/**
 * Returns Color object out of JSON dictionary.
 * @param jsonDictionary JSON dictionary
 * @return deserialized Color object
 */
+(Color*)FromJSON:(NSDictionary*)jsonDictionary;

/**
  * Returns JSON representation out of this Color object.
  * @return JSON representation of this Color returned as NSMutableDictionary
  */
- (NSMutableDictionary*) ToJSON;
@end

@interface BoundingBox:NSObject
{
    @public float x_lowerLeft;
    @public float y_lowerLeft;
    @public float x_upperRight;
    @public float y_upperRight;
}

- (id)init;

- (id)initWithXLowerLeft:(float)x_lowerLeft andYLowerLeft:(float)y_lowerLeft andXUpperRight:(float)x_upperRight andYUpperRight:(float)y_upperRight;

+(BoundingBox*)FromJSON:(NSDictionary*)jsonDictionary;

- (NSMutableDictionary*) ToJSON;

- (NSString*)ToJSONString;
@end

/**
 * Contains rendering information of each element of a makeup feature.
 * A makeup feature can consits of multiple elements with different color, renderign area and/or rendering algorithm.
 *
 * Each ElementTemplate consists of the following:
 * - name: the name of the element
 * - area: the area where rendering will be performed
 * - color: the color of the area where rendering will be performed
 * - pipeline: rendering steps to use when rendering this element.
 *
 * Color is stored in Color class which contains a 4 dimensional vector specifying red, green, blue and alpha channels.
 *
 * Area is specified as a name of the image file that contains a greyscale image specifying how much of the color of this ElementTemplate should be used at each fragment of the mesh used for rendering of the feature containing this ElementTemplate.
 * For each pixel in the area image, if the pixel is white this 100% of color from this element template will be used. If the pixel is black 100% of the color from the previous element will be used. The pixel values in between are lineraly interpolated.
 *
 * Once an effect is created from a feature each element template in the feature is mapped to one element.
 * The previous element of the first element in an effect is the output of the previous effect. If there are no previous effects displayed, the previous element is raw input image without augmentation.
 *
 * The location where area images are stored is @<path to platform dependent assets folder@> + "/Textures/Area/" + FeatureTemplate.type +  ElementTemplate.area where FeatureTemplate.type is type of the feature containing this ElementTemplate and ElementTemplate.area is the area parameter of this ElementTemplate.
 * The location for storing of the color map is @<path to platform dependent assets folder> + "/Textures/" + Element.texture.
 *
 * Since the area image is used as a texture on a mesh used for rendering the feature it will only work correctly if the uv mapping used on the mesh matches the area image.
 * This means that area images can be reused between features only if the features have the same type.
 * Therefore, when specifying a new custom area image, the image file needs to be placed in the subfolder specific to the type of the feature new area was created for.
 *
 * The available meshes are stored in .obj format in the "StreamingAssets/Models" folder of the package. UV maps from these meshes should be used when creating new area textures.
 */
@interface ElementTemplate:NSObject
{
    /**
     * Name of the element
     */
    @public NSString* name;

    /**
     * Name of the image file specifying the area where rendering of this element will be performed.
     */
    @public NSString* area;

    /**
     *  Path used to store color map of the rendered feature.
     */
    @public NSString* texture;

    /**
     * Color of the area where rendering will be performed
     */
    @public Color* color;

    /**
     * Rendering steps to use when rendering the element
     */
    @public PipeTemplate *pipeline;

    /**
     * Some of the feature elements do not use coloring step in their pipelines.
     * Set to true if element uses (RGB) of the color, otherwise it only uses the alpha channel to control intensity.
     */
    @public Boolean useColor;

    /**
     * Some of the feature elements do not use coloring step in their pipelines.
     * Set to true If element should use the provided color map stored in texture member, otherwise it only uses the alpha channel provided in the color member to control intensity.
     */
    @public Boolean useTexture;
}

/**
 * Returns Element object out of JSON dictionary.
 * @param jsonDictionary JSON dictionary
 * @return deserialized Element object
 */
+ (ElementTemplate*) FromJSON: (NSDictionary*) jsonDictionary;

/**
 * Returns JSON representation out of this Element object.
 * @return JSON representation of this Element returned as NSMutableDictionary
 */
- (NSMutableDictionary*) ToJSON;
@end

/**
 * Stores a definition of makeup feature. One makeup feature contains all information required to render virtual makeup:
 * - mesh
 * - area
 * - color
 * - description of blending and filtering algorithms required to augment the camera feed with virtual makeup.
 *
 * One FeatureTemplate instance can contain multiple rendering elements which share the mesh but can have different application area, color or blending and filtering algorithm.
 * The simplest use-case for this class is using one of the predefined template specified in FeatureTemplateRepository obtained from MakeUpEngine::getStyles function and setting the color of each element in elements array.
 *
 * FeatureTemplate is defined by the following:
 * - name: name of the feature. This value is used for identification of different features. Does not have to be unique.
 * - type: type of the feature. Defines a mesh used to represent this feature when rendering.
 * - elements: defines a list of templates of elements required for rendering of this feature. Each element template uses the same mesh defined by the type parameter.
 *
 * The following types are supported:
 * - Lipstick
 * - Lipliner
 * - Mascara
 * - Foundation
 * - Blush
 * - Concealer
 * - Eyeliner
 * - Eyeshadow
 * - Eyebrows_1
 * - Eyebrows_2
 * - Eyebrows_3
 *
 * Each type maps to a single mesh. The meshes used are stored in .obj format in the "Assets/Models" folder. The name of the .obj corresponds to the type name.
 *
 * Additionally, this class provides methods for serialization and deserialization of FeatureTemplate instances from and to JSON format.
 */
@interface FeatureTemplate:NSObject
{
    /**
     * Name of the feature.
     */
    @public NSString* name;

    /**
     * Type of the feature. Specifies one of the predefined meshes to use to represent this feature.
     */
    @public NSString* type;

    /**
     * A list of elements required for rendering this feature.
     */
    @public NSMutableArray<ElementTemplate*> *elements;
}

/**
 * Returns FeatureTemplate object out of JSON string.
 * @param JSON_data JSON string
 * @return deserialized FeatureTemplate object
 */
+ (FeatureTemplate*)FromJSONString:(NSString*)JSON_data;

/**
 * Deserializes FeatureTemplate object from JSON representation.
 * @param jsonDictionary JSON dictionary
 * @return deserialized FeatureTemplate object
 */
+ (FeatureTemplate*) FromJSON: (NSDictionary*) jsonDictionary;

/**
 * Creates JSON representation out of this FeatureTemplate object.
 * @return JSON representation of this FeatureTemplate returned as NSMutableDictionary
 */
- (NSMutableDictionary*)ToJSON;

/**
 * Creates JSON string out of FeatureTemplate object.
 * @param featureTemplate FeatureTemplate object
 * @return JSON representation of this FeatureTemplate returned as string
 */
- (NSString*)ToJSONString;

@end

/**
 * Contains a list of feature templates.
 */
@interface FeatureTemplateRepository:NSObject
{
    NSMutableArray<FeatureTemplate*> *featureTemplates;
}

/**
 * Initializes new FeatureTemplateRepository.
 */
- (id)init;

/**
 * Returns FeatureTemplate object at index i.
 * @param i index
 * @return FeatureTemplate object at given index
 */
- (FeatureTemplate*)Get:(int)i;

/**
 * Returns total number of stored feature templates.
 * @return the number of FeatureTemplate objects in the repository
 */
- (int)GetCount;

/**
 * Adds new FeatureTemplate object to the repository.
 * @param featureTemplate FeatureTemplate object
 */
- (void)Add:(FeatureTemplate*)featureTemplate;

/**
 * Returns FeatureTemplateRepository object out of JSON dictionary.
 * @param JSON_data JSON string
 * @return deserialized FeatureTemplateRepository object
 */
+ (FeatureTemplateRepository*)FromJSON:(NSDictionary*)JSON_data;

/**
 * Returns FeatureTemplateRepository object out of JSON string.
 * @param JSON_data JSON string
 * @return deserialized FeatureTemplateRepository object
 */
+ (FeatureTemplateRepository*)FromJSONString:(NSString*)JSON_data;

/**
 * Returns JSON representation out of this FeatureTemplateRepository object
 * @return JSON representation of this FeatureTemplateRepository returned as NSMutableDictionary
 */
- (NSMutableDictionary*)ToJSON;

/**
 * Returns JSON string out of FeatureTemplateRepository object
 * @param featureTemplateRepository FeatureTemplateRepository object
 * @return JSON representation of this FeatureTemplateRepository returned as string
 */
- (NSString*)ToJSONString;

@end


/**
 * Stores updatable rendering parameters of elements contained in a created makeup effect.
 * Element instances are created from ElementTemplate instances attached to FeatureTemplate and or FinishTemplate used for creating the effect.
 * Each ElementTemplate attached to both FeatureTemplate and FinishTemplate used to create the effect is mapped to a single Element.
 *
 * Element is defined by the following:
 * - name : read-only member used for identification between different elements
 * - color : color of the rendered feature
 * - area : area where the rendering will be performed.
 *
 * Color is stored in Color class which contains a 4 dimensional vector specifying red, green, blue and alpha channels.
 *
 * Area is specified as a name of the image file that contains a greyscale image specifying how much of the color of this Element should be used at each fragment of the mesh used for rendering of the effect containing this Element.
 * For each pixel in the area image, if the pixel is white 100% of color from this element will be used. If the pixel is black 100% of the color from the previous element will be used. The pixel values in between are lineraly interpolated.
 *
 * Once an effect is created from a FeatureTemplate each ElementTemplate in the feature is mapped to one Element.
 * The previous element of the first element in an effect is the output of the previous effect. If there are no previous effects displayed, the previous element is raw input image without augmentation.
 *
 * The location where area images are stored is @<path to platform dependent assets folder@> + "/Textures/Area/" + Effect.type +  Element.area where Effect.type is type of the effect containing this Element and Element.area is the area parameter of this Element.
 * The location for storing of the color map is @<path to platform dependent assets folder> + "/Textures/" + Element.texture.
 *
 * Since the area image is used as a texture on a mesh used for rendering the feature it will only work correctly if the uv mapping used on the mesh matches the area image.
 * This means that area images can be reused between effects only if the effects have the same type.
 * Therefore, when specifying a new custom area image, the image file needs to be placed in the subfolder specific to the type of the effect new area was created for.
 *
 * The available meshes are stored in .obj format in the "Assets/Models" folder of the package. UV maps from these meshes should be used when creating new area images.
 */
@interface Element:NSObject
{
    /**
     * Name of the element
     */
    @private NSString* name;

    /**
     * Name of the image file specifying the area where rendering of this element will be performed.
     */
    @private NSString* area;

    /**
     *  Path used to store color map of the rendered feature.
     */
    @private NSString* texture;

    /**
     * Color of the rendered feature
     */
    @private Color* color;

    /**
     * Some of the feature elements do not use coloring step in their pipelines.
     * Set to true if element uses (RGB) of the color, otherwise it only uses the alpha channel to control intensity.
     */
    @public Boolean useColor;

    /**
     * Some of the feature elements do not use coloring step in their pipelines.
     * Set to true If element should use the provided color map stored in texture member, otherwise it only uses the alpha channel provided in the color member to control intensity.
     */
    @public Boolean useTexture;
}

/**
 * Returns the name of the element
 * @return name of the element
 */
-(NSString*)GetName;

/**
 * Returns the image file name specifying the area of the element.
 * @return area of the element
 */
-(NSString*)GetArea;

 /**
 * Sets the image file name specifying the area of the element.
 * @param area path to texture file specifying area to be used
 */
-(void)SetArea:(NSString*)area;

/**
 * Returns the color of the element
 * @return color of the element
 */
-(Color*)GetColor;

/**
 * Sets the color of the element
 * @param color Color object
 */
-(void)SetColor:(Color*)color;

/**
 * Returns the texture of the element
 * @return texture of the element
 */
-(NSString*)GetTexture;

/**
 * Sets the texture of the element
 * @param texture path to texture file
 */
-(void)SetTexture:(NSString*)texture;


/**
  * Deserializes Element object from JSON representation
  * @param jsonDictionary JSON dictionarx
  * @return deserialized Element object
  */
+ (Element*) FromJSON: (NSDictionary*) jsonDictionary;

/**
 * Returns JSON representation out of Element object
 * @return JSON representation of this Element returned as NSMutableDictionary
 */
- (NSMutableDictionary*)ToJSON;

@end

/**
 * Effect instances are used to store information required to uniquely identify each created makeup entity.
 * Effect instances are created from FeatureTemplate and (optional) FinishTemplate instances passed to MakeUpEngine::CreateMakeup function in the API.
 * Objects of this type are read only.
 * Effect is defined by the following:
 * - id : unique identifier of the effect
 * - type : feature type used to create the effect
 * - name : feature name used to create the effect
 * - elements : a list of elements contained in the effect created from ElementTemplate instances of FeatureTemplate and FinishTemplate used to create this effect.
 */
@interface Effect:NSObject
{
    /**
     * Unique identifier of the effect
     */
    @private unsigned int pk;

    /**
     * Name of the effect
     */
    @private NSString* name;

    /**
     * Makeup type used to create the effect
     */
    @private NSString* type;

    /**
     * A list of elements comprising the effect
     */
    @private NSMutableArray<Element*> *elements;

	@private BoundingBox* boundingBox;
}

/**
 * Returns the id of the effect
 * @return id of the effect
 */
-(unsigned int)GetID;

/**
 * Returns the name of the effect
 * @return name of the effect
 */
-(NSString*)GetName;

/**
 * Returns the type of the effect
 * @return type of the effect
 */
-(NSString*)GetType;

/**
 * Returns the elements required for rendering this effect
 * @return list of the elements
 */
-(NSMutableArray*)GetElements;

/**
 * Returns the bounding box in which this effect is visible
 * @return bounding box
 */
-(NSArray*)GetBoundingBox;

/**
 * Returns the elements required for rendering this effect
 * @return list of the elements
 */
-(void)SetBoundingBox:(NSArray*) boundingBox;

/**
 * Returns Effect object out of JSON string.
 * @param JSON_data JSON string
 * @return deserialized Effect object
 */
+ (Effect*)FromJSONString:(NSString*)JSON_data;

/**
 * Deserializes Effect object from JSON representation.
 * @param jsonDictionary JSON dictionary
 * @return deserialized Effect object
 */
+ (Effect*) FromJSON: (NSDictionary*) jsonDictionary;

/**
 * Creates JSON representation out of this Effect object.
 * @return JSON representation of this Effect returned as NSMutableDictionary
 */
- (NSMutableDictionary*)ToJSON;

/**
 * Creates JSON string out of this Effect object.
 * @return JSON representation of this Effect returned as string
 */
- (NSString*)ToJSONString;

@end
