#pragma once
#pragma warning(disable : 4251) // ignore export templates warning

#include "FrameProvider/IFrameObserver.h"
#include "FrameProvider/IPromiseResolver.h"
#include "FrameProvider/Image.h"
#include "FrameProvider/TripleBuffer.h"
#include "FrameProvider/frameprovider_export.h"
#include <condition_variable>
#include <mutex>
#include <vector>

#ifdef __EMSCRIPTEN__
constexpr bool isSingleThreaded = true;
#else
constexpr bool isSingleThreaded = false;
#endif

namespace fpr {

/**
 * Interface used to provide image data to MakeupEngine for processing.
 * API users should obtain a reference to this interface by using MakeupEngine::GetFrameProvider function.
 */
class FRAMEPROVIDER_EXPORT FrameProvider : public IPromiseResolver,
                                           public TripleBufferObserver<CamIL::Image> {
    public:
    /**
     * Enqueues image for processing, replacing the last queued Image.
     * When processing is triggered the engine obtaines only the last queued frame.
     * @param newFrame Image for processing
     * @return sequential frame index
     */
    long AddFrame(const CamIL::Image& newFrame);

    /**
     * Enqueues image for processing, replacing the last queued Image.
     * When processing is triggered the engine obtaines only the last queued frame.
     * @param frame Image for processing
     * @return sequential frame index
     */
    long AddFrame(CamIL::Image&& frame);

    /**
     * Returns previous input image. Use this method to reuse memory and avoid
     * unnecessary allocations/dealocations that could cause a performance hit.
     *
     * Use this only when new frame has the same format, width and height as the returned image.
     * If format, width or height don't match, create a new CamIL::Image.
     */
    CamIL::Image& ReuseInputFrame();

    /**
     * Returns the latest ready frame from camera.
     */
    CamIL::Image& GetFrame();

    /**
     * Subscribes to new frames.
     * @param observer Object reference to receive notifications when new frame is enqueued.
     */
    void Subscribe(IFrameObserver& observer);

    /**
     * Unsubscribes from new frames.
     * @param observer Subscribed observer reference you wish to cancel.
     */
    void Unsubscribe(const IFrameObserver& observer);

    /**
     * Clears frames.
     */
    void Clear();

    private:
    void Resolve() override;
    void Notify();
    void UpdateCounter();
    virtual void onOverwrite(CamIL::Image& buffer) override;

    TripleBuffer<CamIL::Image> tripleBuffer = { this, isSingleThreaded };
    std::vector<IFrameObserver*> observers;
    int nResolvedObservers = 0;
    bool newFrame = false;

    mutable std::mutex observersMutex;
    std::condition_variable_any observersCondition;
    long frameId = 0;
};

} // namespace fpr
