#pragma once
#include "FrameProvider/Format.h"
#include "FrameProvider/Plane.h"
#include "FrameProvider/frameprovider_export.h"
#include <memory>

#ifdef ANDROID
#include <android/log.h>

#define TAG "MY_TAG"

#define LOGE(...) __android_log_print(ANDROID_LOG_ERROR, TAG, __VA_ARGS__)
#define LOGW(...) __android_log_print(ANDROID_LOG_WARN, TAG, __VA_ARGS__)
#define LOGI(...) __android_log_print(ANDROID_LOG_INFO, TAG, __VA_ARGS__)
#define LOGD(...) __android_log_print(ANDROID_LOG_DEBUG, TAG, __VA_ARGS__)
#endif

namespace CamIL {

/**
 * Represents an image retrieved from camera.
 */
class FRAMEPROVIDER_EXPORT Image {
    public:
    Image();

    /**
     * Creates a new image.
     * @param width Image width.
     * @param height Image height.
     * @param format Image format.
     */
    Image(int width, int height, IFormat format);

    /**
     * Creates a new image.
     * @param width Image width.
     * @param height Image height.
     * @param format Image format. The only formats currently supported for both
     * tracking and rendering are: BGRA, LUMINANCE, RGB, RGBA.
     */
    Image(int width, int height, EFormat format);

    /**
     * Creates a new image.
     * @param width Image width.
     * @param height Image height.
     * @param format Image format.
     * @param srcArray Image data.
     */
    Image(int width, int height, IFormat format, void* const* srcArray);

    /**
     * Creates a new image.
     * @param width Image width.
     * @param height Image height.
     * @param format Image format. The only formats currently supported for both
     * tracking and rendering are: BGRA, LUMINANCE, RGB, RGBA.
     * @param srcArray Image data.
     */
    Image(int width, int height, EFormat format, void* const* srcArray);

    Image(const Image&);
    Image& operator=(Image const&);
    Image& operator=(Image&&);
    Image(Image&&);

    /**
     * Duplicates image and it's buffer.
     */
    Image* Clone() const;

    /**
     * Returns the buffer size.
     */
    int GetSize() const;

    /**
     * Gets the number of planes the image contains.
     */
    int GetPlaneCount() const;

    /**
     * Gets the plane on the given index.
     */
    Plane& GetPlane(int index) const;

    /**
     * Gets the format description object.
     */
    IFormat GetFormat() const;

    /**
     * Gets the plane of the given name.
     */
    Plane& GetPlane(const char* plane) const;

    /**
     * Represents the contiguous block of memory the image is stored in.
     */
    class ContiguousMultiPlaneBuffer {
        public:
        ContiguousMultiPlaneBuffer(int size);

        /**
         * Gets the pointer to start of the image data.
         */
        void* GetBuffer();

        /**
         * Duplicates the image data in another part of contiguous memory.
         */
        ContiguousMultiPlaneBuffer* Clone();

        private:
        std::vector<char> storage;
    };

    private:
    std::vector<std::shared_ptr<Plane>> planes;
    std::shared_ptr<ContiguousMultiPlaneBuffer> storage;

    std::unique_ptr<IFormat> format;

    static int calculateTotalStorage(int width, int height, IFormat const& format);
};

} // namespace CamIL
