#pragma once

#include <array>
#include <condition_variable>
#include <mutex>

namespace fpr {
template <class T>
class TripleBufferObserver {
    public:
    virtual void onOverwrite(T& buffer) = 0;
};

template <class T>
class TripleBuffer {
    public:
    TripleBuffer(TripleBufferObserver<T>* observer = nullptr, bool isSingleThreaded = true)
    : observer(observer), isSingleThreaded(isSingleThreaded) {}

    /**
     * Returns the latest ready frame from camera.
     * Returns a pointer to internally allocated memory, do not delete it.
     */
    T& Read() { return buffers[ub]; }

    /**
     * Writes a frame to the write buffer and swaps write buffer with the
     * third buffer.
     */
    void Write(const T& input) {
        std::lock_guard<std::mutex> lock(buffersMutex);

        if(observer != nullptr) {
            observer->onOverwrite(buffers[wb]);
        }

        buffers[wb] = input;

        SwapWrite();
    }

    void Write(T&& input) {
        std::lock_guard<std::mutex> lock(buffersMutex);

        if(observer != nullptr) {
            observer->onOverwrite(buffers[wb]);
        }

        buffers[wb] = std::move(input);

        SwapWrite();
    }

    /**
     * Swaps read buffer with the third buffer.
     */
    void SwapRead() {
        if(isSingleThreaded && !newFrame) {
            return;
        }

        std::unique_lock<std::mutex> lock(indicesMutex);
        auto isNewFrame = [this]() { return this->newFrame; };

        if(!isSingleThreaded) {
            indicesCondition.wait(lock, isNewFrame);
        }

        std::swap(ub, rb);
        newFrame = false;
    }

    void Clear() {
        buffers[wb] = T();
        buffers[rb] = T();
        buffers[ub] = T();
    }

    auto& AccessWriteBuffer() { return buffers[wb]; }

    private:
    void SwapWrite() {
        std::unique_lock<std::mutex> lock(indicesMutex);

        std::swap(wb, rb);

        newFrame = true;

        lock.unlock();

        indicesCondition.notify_one();
    }

    std::array<T, 3> buffers;

    TripleBufferObserver<T>* observer = nullptr;

    /**
     * Mutex used for guarding access to buffer indices.
     */
    mutable std::mutex indicesMutex;

    /**
     * Condition used for waiting until a new frame is available.
     */
    std::condition_variable indicesCondition;

    /**
     * Mutex used for guarding access to buffers.
     */
    mutable std::mutex buffersMutex;

    // buffer indices
    char wb = 0;
    char rb = 1;
    char ub = 2;

    /**
     * True if a new frame from camera is available, false otherwise.
     */
    bool newFrame = false;

    /**
     * True if single thread
     */
    bool isSingleThreaded = true;
};

} // namespace fpr
