#import "FeatureTemplate.h"
#import "InternalMakeUpObserver.h"

#ifdef FRAMEWORK
#import "FrameProvider/FrameProvider.h"
#endif

/**
 * Used to forward platform specific lifecycle events to MakeupEngine.
 * The user is responsible to invoke the methods of this class when appropriate
 *lifecycle event is raised by the OS.
 **/
@interface IApplicationLifecycleHandler : NSObject {
}

/**
 * Invoke when UIApplicationDelegate::applicationWillResignActive event is
 *invoked.
 **/
- (void)applicationWillResignActive;

/**
 * Invoke when UIApplicationDelegate::applicationDidBecomeActive event is
 *invoked.
 **/
- (void)applicationDidBecomeActive;

/**
 * Invoke when UIApplicationDelegate::applicationWillEnterForeground event is
 *invoked.
 **/
- (void)applicationWillEnterForeground;

/**
 * Invoke when UIApplicationDelegate::applicationDidEnterBackground event is
 *invoked.
 **/
- (void)applicationDidEnterBackground;

@end

/**
 * Base API of the SDK functionality.
 * This class serves two functions:
 * - forwarding of method calls for makeup effect manipulation to Unity via
 * UnitySendMessage
 * - deserialization of FeatureTemplateRepository and FinishTemplateRepository
 * from JSON. This class is responsible for creating, displaying, updating,
 * hiding and removing makeup features.
 */
@interface MakeUpEngine : NSObject {
}

#ifndef FRAMEWORK
/**
 * Constructor of the MakeUpEngine
 * @param licensePath Path to the license
 */
- (id)init:(const char *)licensePath;
#endif

#ifdef FRAMEWORK

/**
 * Constructor of the MakeUpEngine
 * @param licensePath Path to the license
 * @param frameProvider instance of frame provider used to provide input images
 * to the SDK
 */
- (id)init:(const char *)licensePath:(fpr::FrameProvider *)frameProvider;

/**
 * Initializes the Unity engine
 */
- (void)InitUnity;
#endif

/**
 * Attaches observer to the engine. IMakeUpObserver is used to obtain event
 * notifications from the engine.
 * @param observer Instance of IMakeUpObserver
 */
- (void)AddObserver:(id<IMakeUpObserver>)observer;

- (FeatureTemplateRepository *)LoadFeatureTemplates;

/**
 * Returns loaded feature template repository containing predefined features.
 * Each of the predefined feature templates specify a standard renderfing
 * procedure for each of the makeup features supported by the engine.
 */
- (FeatureTemplateRepository *)GetFeatures;

/**
 * Creates makeup effect from given feature and finish templates.
 * Returns the id of the created effect via IMakeupObserver interface.
 * @param featureTemplate FeatureTemplate object
 * @param finishTemplate FinishTemplate object
 */
- (void)CreateMakeup:(FeatureTemplate *)
     featureTemplate:(FinishTemplate *)finishTemplate;

/**
 * Creates makeup effect from given feature and finish templates.
 * Additionally, this function takes an array of NSNumbers specifying a bounding
 * box in the following way:
 * - Lower left x coordinate is the first element of the array
 * - Lower left y coordinate is the second element of the array
 * - Upper right x coordinate is the third element of the array
 * - Upper right y coordinate is the fourth element of the array
 * Bounding box is set in normalized image coordinates where point (0.0, 0.0) is
 * the lower left corner of the output image while point (1.0, 1.0) is the upper
 * right corner of the output image. This effect will be visible only when its
 * location in the output image is inside of the bounding box. The default
 * bounding box is contains the whole output image. If the input array has less
 * than 4 elements, the default bounding box will be used. Returns the id of the
 * created effect via IMakeupObserver interface.
 * @param featureTemplate FeatureTemplate object
 * @param finishTemplate FinishTemplate object
 * @param boundingBox Quuartet specifying bounding box in which this effect is
 * visible
 */
- (void)CreateMakeup:(FeatureTemplate *)
     featureTemplate:(FinishTemplate *)finishTemplate
                    :(NSArray *)boundingBox;

/**
 * Displays given makeup effect.
 * @param effect makeup effect
 */
- (void)DisplayMakeup:(Effect *)effect;

/**
 * Updates given makeup effect.
 * @param effect makeup effect
 */
- (void)UpdateMakeup:(Effect *)effect;

/**
 * Destroys all created makeup effects.
 */
- (void)ClearMakeup;

/**
 * Hides given makeup effect.
 * @param effect makeup effect
 */
- (void)HideMakeUp:(Effect *)effect;

/**
 * Destroys given makeup effect.
 * @param effect makeup effect
 */
- (void)RemoveMakeUp:(Effect *)effect;

/**
 * Defines which camera will be used.
 * @param id camera id
 */
- (void)SetCameraDevice:(unsigned int)id;

/**
 * Returns loaded finish template repository containing predefined finishes.
 * Each of the predefined finish templates specify a standard renderfing
 * procedure for each of the finishes supported by the engine.
 */
- (FinishTemplateRepository *)GetFinishes;

- (FinishTemplateRepository *)LoadFinishTemplates;

- (IApplicationLifecycleHandler *)GetLifecycleHandler;

- (UIView *)GetView;

@end
